// Basemenu.cc for Openbox
// Copyright (c) 2001 Sean 'Shaleh' Perry <shaleh@debian.org>
// Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifdef    HAVE_CONFIG_H
#  include "../config.h"
#endif // HAVE_CONFIG_H

#include "Menu.h"
#include "i18n.h"
#include "openbox.h"
#include "Screen.h"

Menu::Menu(BScreen &scrn) : openbox(scrn.getOpenbox()), screen(scrn),
  image_ctrl(scrn.getImageControl())
{
  display = openbox.getXDisplay();
  parent = NULL;
  alignment = AlignDontCare;
  _title_vis = _movable = _hide_tree = true;
  _shifted = _internal_menu = _moving = _torn = _visible = false;
  _which_sub = _which_press = _which_sbl = -1;
  _frame_pixmap = _title_pixmap = _hilite_pixmap = _sel_pixmap = None;
  _label = "";
  _sublevels = _persub = _minsub = 0;
      
  unsigned int w, h;
  int bevel_w = screen.getBevelWidth();
  if (i18n->multibyte())
    w = screen.getMenuStyle()->t_fontset_extents->max_ink_extent.height +
      (bevel_w  * 2);
  else
    w = screen.getMenuStyle()->t_font->ascent +
      screen.getMenuStyle()->t_font->descent + (bevel_w * 2);

  MenuStyle *style = screen.getMenuStyle();
  if (i18n->multibyte())
    h = style->f_fontset_extents->max_ink_extent.height + (bevel_w);
  else
    h = style->f_font->ascent + style->f_font->descent + (bevel_w);

  _area = Rect(0, 0, w, h);
  _move = Point(0, 0);
  _shift = Point(0, 0);
  _title_h = _frame_h = w;
  _item_sz(w, h);

  unsigned long attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
            CWColormap | CWOverrideRedirect | CWEventMask;
  XSetWindowAttributes attrib;
  attrib.background_pixmap = None;
  attrib.background_pixel = attrib.border_pixel =
          screen.getBorderColor()->getPixel();
  attrib.colormap = screen.getColormap();
  attrib.override_redirect = True;
  attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
                      ButtonMotionMask | ExposureMask;

  menu.window =
    XCreateWindow(display, screen.getRootWindow(), menu.x, menu.y, menu.width,
      menu.height, screen.getBorderWidth(), screen.getDepth(),
                  InputOutput, screen.getVisual(), attrib_mask, &attrib);
  openbox.saveMenuSearch(menu.window, this);

  attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel | CWEventMask;
  attrib.background_pixel = screen.getBorderColor()->getPixel();
  attrib.event_mask |= EnterWindowMask | LeaveWindowMask;

  menu.title =
    XCreateWindow(display, menu.window, 0, 0, menu.width, menu.height, 0,
      screen.getDepth(), InputOutput, screen.getVisual(),
      attrib_mask, &attrib);
  openbox.saveMenuSearch(menu.title, this);

  attrib.event_mask |= PointerMotionMask;
  menu.frame = XCreateWindow(display, menu.window, 0,
           menu.title_h + screen.getBorderWidth(),
           menu.width, menu.frame_h, 0,
           screen.getDepth(), InputOutput,
           screen.getVisual(), attrib_mask, &attrib);
  openbox.saveMenuSearch(menu.frame, this);

  // even though this is the end of the constructor the menu is still not
  // completely created.  items must be inserted and it must be update()'d
}
